@@ -1,22 +1,24 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-import os
+import os, logging
 import pandas as pd
 import numpy as np
 from tqdm import tqdm
 import warnings, multiprocessing
+from astropy.utils.console import ProgressBar
 from .diffdata import DiffData
 
+logger = logging.getLogger(__name__)
+
 
 def run_forcephotometry(
     target,
     download_files=True,
     nprocess=4,
-    verbose=True,
     filecheck=True,
     ignore_warnings=False,
-    **kwargs
+    **kwargs,
 ):
     """ """
     fp = ForcePhotometry.from_name(target)
@@ -24,7 +26,7 @@ def run_forcephotometry(
     if download_files:
         fp.io.download_data(nprocess=nprocess)
     fp.load_filepathes(filecheck=filecheck, ignore_warnings=ignore_warnings)
-    fp.run_forcefit(update_diffdata=False, verbose=verbose, **kwargs)
+    fp.run_forcefit(update_diffdata=False, **kwargs)
 
 
 class ForcePhotometry:
@@ -36,6 +38,7 @@ class ForcePhotometry:
     def __init__(self, ztftarget):
         """ """
         self.io = ztftarget
+        self.logger = logging.getLogger(__name__)
 
     @classmethod
     def from_name(cls, name):
@@ -57,19 +60,21 @@ class ForcePhotometry:
     # -------- #
     # LOADDER  #
     # -------- #
-    def load_metadata(self, **kwargs):
+    def load_metadata(self, clean=True, **kwargs):
         """ """
         if not hasattr(self.io, "marshal"):
             self.io.load_marshal(**kwargs)
-        self.io.load_metadata()
+        self.meta = self.io.load_metadata(clean=clean)
 
-    def load_filepathes(self, **kwargs):
+    def load_filepathes(self, download_dir=None, **kwargs):
         """
         **kwargs eventually goes to ztfquery.Query.get_local_data()
                  -> exists=True, indexes=None, filecheck=True, ignore_warnings=False, etc.
 
         """
-        _filepaths = self.io.get_diffimg_forcepsf_filepath(**kwargs)
+        _filepaths = self.io.get_diffimg_forcepsf_filepath(
+            download_dir=download_dir, **kwargs
+        )
 
         # clean list of missing psf image entries (will be None)
         self._filepathes = [f for f in _filepaths if f[1] is not None]
@@ -84,7 +89,8 @@ class ForcePhotometry:
             filename = LOCALDATA + "/%s.csv" % self.io.name
 
         oldmask = os.umask(0o002)
-        self._data_forcefit.to_csv(filename, index=False, mode=mode)
+        if len(self._data_forcefit) > 0:
+            self._data_forcefit.to_csv(filename, index=False, mode=mode)
 
     # -------- #
     # FITTER   #
@@ -94,7 +100,6 @@ class ForcePhotometry:
         indexes=None,
         update_diffdata=False,
         store=False,
-        verbose=True,
         no_badsub=False,
         force_refit=False,
         nprocess=4,
@@ -118,16 +123,14 @@ class ForcePhotometry:
             previous_results = pd.DataFrame(columns=["filename"])
 
         dataout = {}
-        if verbose:
-            print("Starting run_forcefit() for %d image differences" % len(indexes))
-
-        from astropy.utils.console import ProgressBar
+        self.logger.info(
+            f"Starting run_forcefit() for {len(indexes)} image differences"
+        )
 
         index_count = len(indexes)
         _coords = [self.io.get_coordinate()] * index_count
         _update_diffdata = [update_diffdata] * index_count
         _no_badsub = [no_badsub] * index_count
-        _verbose = [verbose] * index_count
         _previous_results = [previous_results] * index_count
         _force_refit = [force_refit] * index_count
 
@@ -145,7 +148,6 @@ class ForcePhotometry:
                             _coords,
                             _update_diffdata,
                             _no_badsub,
-                            _verbose,
                             _previous_results,
                             _force_refit,
                         ),
@@ -165,9 +167,8 @@ class ForcePhotometry:
 
         else:
             for i in tqdm(indexes):
-                if verbose:
-                    print("running %d " % i)
-                    print(self.filepathes[i][0].split("/")[-1])
+                self.logger.debug("running %d " % i)
+                self.logger.debug(self.filepathes[i][0].split("/")[-1])
 
                 filename_modified = self.filepathes[i][0].split("/")[-1][:-26] + ".fits"
                 query = previous_results.query("filename == @filename_modified")
@@ -181,17 +182,15 @@ class ForcePhotometry:
 
                 # Now do the actual check
                 if len(query) == 40 and not force_refit:
-                    if verbose:
-                        print("not fitting %d " % i)
+                    self.logger.debug("not fitting %d " % i)
                     dataout[i] = query
 
                 else:
-                    if verbose:
-                        print("fitting %d " % i)
+                    self.logger.debug("fitting %d " % i)
                     diffdata = self.get_ith_diffdata(i, update=update_diffdata)
                     has_nan = np.any(np.isnan(diffdata.diffimg))
                     if has_nan and no_badsub:
-                        print("NaNs in the image, skipped")
+                        self.logger.info("NaNs in the image, skipped")
                     else:
                         try:
                             with warnings.catch_warnings():
@@ -201,16 +200,17 @@ class ForcePhotometry:
                                 dataout[i] = {**fitresults, **datainfo}
                                 dataout[i]["data_hasnan"] = has_nan
                         except ValueError:
-                            if verbose:
-                                warnings.warn(
-                                    "Shape of diffimg and psfimg do not correspond (index: %d). Skipping."
-                                    % (i)
-                                )
+                            self.logger.debug(
+                                "Shape of diffimg and psfimg do not correspond (index: %d). Skipping."
+                                % (i)
+                            )
                             pass
                     del diffdata
                 gc.collect()
 
             self._data_forcefit = pd.DataFrame(dataout).T
+
+            self.logger.info(f"Fitted {len(self._data_forcefit)} datapoints.")
             if store:
                 self.store()
 
@@ -254,7 +254,6 @@ class ForcePhotometry:
             coords,
             update_diffdata,
             no_badsub,
-            verbose,
             previous_results,
             force_refit,
         ) = args
@@ -271,19 +270,17 @@ class ForcePhotometry:
 
         # Now do the actual check
         if len(query) == 40 and not force_refit:
-            if verbose:
-                print("not fitting %d " % index)
+            self.logger.debug("not fitting %d " % index)
             dataout = query
 
         else:
-            if verbose:
-                print("fitting %d " % index)
-                print(filepath[0].split("/")[-1])
+            logger.debug("fitting %d " % index)
+            logger.debug(filepath[0].split("/")[-1])
+
             diffdata = DiffData(*filepath, coords)
             has_nan = np.any(np.isnan(diffdata.diffimg))
             if has_nan and no_badsub:
-                if verbose:
-                    print("NaNs in the image, skipped")
+                self.logger.debug("NaNs in the image, skipped")
                 return None
             else:
                 try:
@@ -294,11 +291,10 @@ class ForcePhotometry:
                         dataout = {**fitresults, **datainfo}
                         dataout["data_hasnan"] = has_nan
                 except ValueError:
-                    if verbose:
-                        warnings.warn(
-                            "Shape of diffimg and psfimg do not correspond (index: %d). Skipping."
-                            % (index)
-                        )
+                    logger.debug(
+                        "Shape of diffimg and psfimg do not correspond (index: %d). Skipping."
+                        % (index)
+                    )
                     return None
             del diffdata
 
@@ -340,7 +336,7 @@ class ForcePhotometry:
                 yerr=err[flag],
                 ls="None",
                 ecolor="0.6",
-                **prop
+                **prop,
             )
 
         ax.axhline(0, ls="--", color="0.5")
